#ifndef KGIO_H
#define KGIO_H

#include <ruby.h>
#ifdef HAVE_RUBY_IO_H
#  include <ruby/io.h>
#else
#  include <rubyio.h>
#endif
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <assert.h>
#include <netdb.h>

#include "ancient_ruby.h"

struct io_args {
	VALUE io;
	VALUE buf;
	char *ptr;
	long len;
	int fd;
};

void init_kgio_wait(void);
void init_kgio_read_write(void);
void init_kgio_accept(void);
void init_kgio_connect(void);
void init_kgio_autopush(void);
void init_kgio_poll(void);
void init_kgio_tryopen(void);

void kgio_autopush_accept(VALUE, VALUE);
void kgio_autopush_recv(VALUE);
void kgio_autopush_send(VALUE);

VALUE kgio_call_wait_writable(VALUE io);
VALUE kgio_call_wait_readable(VALUE io);
#if defined(HAVE_RB_THREAD_BLOCKING_REGION) && defined(HAVE_POLL)
#  define USE_KGIO_POLL
#endif /* USE_KGIO_POLL */

#ifndef HAVE_RB_UPDATE_MAX_FD
#  define rb_update_max_fd(fd) for (;0;)
#endif

/*
 * 2012/12/13 - Linux 3.7 was released on 2012/12/10 with TFO.
 * Headers distributed with glibc will take some time to catch up and
 * be officially released.  Most GNU/Linux distros will take a few months
 * to a year longer. "Enterprise" distros will probably take 5-7 years.
 * So keep these until 2017 at least...
 */
#ifdef __linux__
#  ifndef MSG_FASTOPEN
#    define MSG_FASTOPEN	0x20000000 /* for clients */
#  endif
#  ifndef TCP_FASTOPEN
#    define TCP_FASTOPEN	23 /* for listeners */
#  endif
   /* we _may_ have TFO support */
#  define KGIO_TFO_MAYBE (1)
#else /* rely entirely on standard system headers */
#  define KGIO_TFO_MAYBE (0)
#endif

extern unsigned kgio_tfo;

#endif /* KGIO_H */
